/*
**    TP 5ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "spring_drawer.hpp"
#include "../libmesh/opengl_drawer.hpp"

#include <cmath>

#include <GL/gl.h>
#include <GL/glu.h>

namespace cpe
{
spring_drawer::spring_drawer()
{
}

spring_drawer::spring_drawer(const cpe::mesh& sphere_local)
    :sphere(sphere_local)
{}


void spring_drawer::draw_spring(const cpe::vec3& p0,const cpe::vec3& p1)
{
    if(sphere.number_of_vertices()==0)
    {
        std::cout<<"Warning: No sphere initialized in "<<__FILE__<<","<<__FUNCTION__<<std::endl;
    }

    glPushMatrix();
    glTranslated(p0[0],p0[1],p0[2]);
    cpe::opengl_drawer::draw(sphere);
    glPopMatrix();

    glPushMatrix();
    glTranslated(p1[0],p1[1],p1[2]);
    cpe::opengl_drawer::draw(sphere);
    glPopMatrix();

    draw_spring_curve(p0,p1);
}

void spring_drawer::draw_spring_curve(const cpe::vec3& p0,const cpe::vec3& p1)
{
    std::vector<double> curve;
    unsigned int N=500;
    double R=0.01;
    double L=(p1-p0).norm();
    for(unsigned int k=0;k<N;++k)
    {
        double u=double(k)/double(N-1);
        double f=15;
        double x=R*cos(f*2*M_PI*u);
        double y=R*sin(f*2*M_PI*u);
        double z=L*u;
        curve.push_back(x);curve.push_back(y);curve.push_back(z);
    }

    //rotation
    try
    {


        vec3 u=(p1-p0).normalized();
        vec3 u0=vec3(0,0,1);
        double cos_t=u.dot(u0);
        double sin_t=std::sqrt(std::abs(1.0-cos_t*cos_t));

        vec3 axis=u0.cross(u);
        float epsilon=1e-5;
        float n_axis=axis.norm();
        if(n_axis<epsilon)
            axis=vec3(1,0,0);
        else
            axis/=n_axis;

        for(unsigned int k=0;k<N;++k)
        {
            vec3 p=vec3(curve[3*k+0],curve[3*k+1],curve[3*k+2]);
            vec3 p_rot=p*cos_t + (axis.cross(p))*sin_t + axis*(axis.dot(p))*(1-cos_t);

            curve[3*k+0]=p_rot[0];
            curve[3*k+1]=p_rot[1];
            curve[3*k+2]=p_rot[2];
        }


        vec3 e1=vec3(1,0,0);
        vec3 e2=vec3(0,1,0);
        vec3 t1=e1*cos_t + (axis.cross(e1))*sin_t + axis*(axis.dot(e1))*(1-cos_t);

        vec3 proj_t1=t1.dot(e1)*e1+t1.dot(e2)*e2;
        double cos_p=proj_t1.normalized().dot(e1);
        double sin_p=-std::sqrt(std::abs(1.0-cos_p*cos_p));

        vec3 rotated_e1=e1*cos_p + (vec3(0,0,-1).cross(e1))*sin_p + vec3(0,0,-1)*(vec3(0,0,-1).dot(e1))*(1-cos_p);
        bool is_inverted=false;
        if(rotated_e1[1]*proj_t1[1]>0)
        {
            is_inverted=true;
        }
        if(is_inverted==true)
            sin_p=-sin_p;



        for(unsigned int k=0;k<N;++k)
        {
            curve[3*k+0]+=p0[0];
            curve[3*k+1]+=p0[1];
            curve[3*k+2]+=p0[2];
        }


        glDisable(GL_LIGHTING);
        glLineWidth(2.0);
        glColor3d(0.2,0.2,0.2);
        glBegin(GL_LINE_STRIP);
        for(unsigned int k=0;k<N;++k)
        {
            glVertex3d(curve[3*k+0],curve[3*k+1],curve[3*k+2]);
        }
        glEnd();
        glLineWidth(1.0);

    }
    catch(cpe::exception_cpe& e)
    {

    }

}

void spring_drawer::draw_spring_force(const cpe::vec3& p0,const cpe::vec3& force,const float& magnification)
{
    glDisable(GL_LIGHTING);
    const cpe::vec3 f=p0+magnification*force;

    glBegin(GL_LINES);
    glVertex3fv(p0.pointer());
    glVertex3fv(f.pointer());
    glEnd();

}

}
