/*
**    TP 5ETI CPE Lyon
**    Copyright (C) 2012 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef MESH_SPRING_DRAWER_HPP
#define MESH_SPRING_DRAWER_HPP

#include "../libmesh/mesh.hpp"



namespace cpe
{

/** \brief Helper class to draw a spring structure */
class spring_drawer
{
public:

    // ********************************************* //
    // ********************************************* //
    //  Constructor
    // ********************************************* //
    // ********************************************* //

    /** Empty constructor */
    spring_drawer();
    /** Direct constructor storing a mesh sphere */
    spring_drawer(const cpe::mesh& sphere);

    // ********************************************* //
    // ********************************************* //
    //  Drawing method
    // ********************************************* //
    // ********************************************* //

    /** \brief Draw a spring between two positions */
    void draw_spring(const cpe::vec3& p0,const cpe::vec3& p1);

    /** \brief Draw a vector of force at a given position */
    void draw_spring_force(const cpe::vec3& p0,const cpe::vec3& force,const float& magnification);

private:

    /** \brief Helper class to draw the oscillating spring */
    void draw_spring_curve(const cpe::vec3& p0,const cpe::vec3& p1);

    /** \brief Internal storage of a mesh structure */
    cpe::mesh sphere;


};

}

#endif
