
/** TP 5ETI - CPE Lyon - 2012/2013 */


#ifndef SCENE_HPP
#define SCENE_HPP

#include "../lib3d/matrix3.hpp"
#include "../lib3d/vec3.hpp"
#include "../libmesh/mesh.hpp"
#include "../libspring/spring_drawer.hpp"

#include <QTime>
#include <vector>

class glwidget;

namespace cpe
{
class debug_mesh;
}

class scene
{


public:

    scene();



    /** \brief Method called only once at the beginning (load off files ...) */
    void load_model();
    /** \brief Method called at every frame */
    void draw_scene();







    //set glwidget pointer
    void set_glwidget(glwidget *p_widget);

    //set debug_mesh pointer
    void set_debug_mesh(cpe::debug_mesh *p_debug_mesh);

private:



    //position of the particle 0
    cpe::vec3 position_0;
    //speed of the particle 0
    cpe::vec3 speed_0;

    //position of the particle 1
    cpe::vec3 position_1;
    //speed of the particle 1
    cpe::vec3 speed_1;

    //rest length of the spring
    float rest_length;
    //spring stiffness
    float stiffness;
    //attenuation coefficient (fluid damping)
    float damping;

    //time step
    float time_step;


    // compute force
    cpe::vec3 compute_force(float stiffness,float rest_length,const cpe::vec3& position_start,const cpe::vec3& position_end);
    //perform integration step
    void integration_step(cpe::vec3 *position_pointer,cpe::vec3 *speed_pointer,const cpe::vec3& force,float mu,float dt);


    //timing parameters
    QTime timer; //Qt timer
    double t_current; //current time
    double t_previous;//previous time
    double t_diff; //difference between current and previous
    int framerate; //time difference between integration

    //sphere mesh (to draw spring)
    cpe::mesh sphere;

    //constant gravity attraction
    cpe::vec3 gravity;

    //mesh storing the ground
    cpe::mesh ground;
    //id for texture
    unsigned int texture_id_ground;



    //helper class to draw spring
    cpe::spring_drawer spring_draw;

    //helper to draw axes
    void draw_axes();

    //helper to generate meshes
    void generate_meshes();

    //is animation in pause
    bool is_pause;


    //widget and helpers
    glwidget *p_widget;
    cpe::debug_mesh *p_debug_mesh;

};

#endif
