
/** TP 5ETI - CPE Lyon - 2013 */


#ifndef SCENE_HPP
#define SCENE_HPP

#include "../lib3d/matrix3.hpp"
#include "../lib3d/vec3.hpp"
#include "../libmesh/mesh.hpp"
#include "../libspring/spring_drawer.hpp"
#include "../libspring/spring_mesh.hpp"

#include <QTime>

#include <vector>

class glwidget;

namespace cpe
{
class debug_mesh;
}

class scene
{


public:

    scene();



    /** \brief Method called only once at the beginning (load off files ...) */
    void load_model();
    /** \brief Method called at every frame */
    void draw_scene();







    //set glwidget pointer
    void set_glwidget(glwidget *p_widget);

    //set debug_mesh pointer
    void set_debug_mesh(cpe::debug_mesh *p_debug_mesh);



    //get/set method for internal parameters
    void set_is_pause(bool is_enable);
    void set_is_grid(bool is_enable);
    void set_is_surface(bool is_enable);
    void set_stiffness_structural(float K_struct);
    void set_stiffness_shear(float K_shear);
    void set_stiffness_bending(float K_bending);
    void set_size_cloth(int s);
    void set_time_step_cloth(float dt);
    void set_damping_cloth(float mu);
    void set_steps(int steps);
    void set_delta_grid(int d_grid);
    void restart();
    void set_wind(const cpe::vec3& w);

private:

    //the cloth mesh
    cpe::spring_mesh cloth;


    //timing parameters
    QTime timer; //Qt timer
    double t_current; //current time
    double t_previous;//previous time
    double t_diff; //difference between current and previous
    int framerate; //time difference between integration

    //sphere mesh (to draw spring)
    cpe::mesh sphere;

    cpe::mesh ball;

    //constant gravity attraction
    cpe::vec3 gravity;

    //mesh storing the ground
    cpe::mesh ground;
    //id for ground texture
    unsigned int texture_id_ground;
    //id for cloth texture
    unsigned int texture_id_cloth;


    //helper to draw axes
    void draw_axes();

    //helper to generate meshes
    void generate_meshes();




    //widget and helpers
    glwidget *p_widget;
    cpe::debug_mesh *p_debug_mesh;


    //is animation in pause
    bool is_pause;
    bool is_grid;
    bool is_surface;

    //number of integration per drawing step
    unsigned int steps_cloth;
    //interval of vertices between two lines
    unsigned int delta_grid;

    //draw the cloth as a grid structure
    void draw_grid(const cpe::spring_mesh& mesh);


};

#endif
